----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:50:02 11/13/2008 
-- Design Name: 
-- Module Name:    ALU1bit_MSB - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity ALU1bit_MSB is
	Port (
		ALU1bit_MSB_a        : in  STD_LOGIC;
      ALU1bit_MSB_b        : in  STD_LOGIC;
		ALU1bit_MSB_less     : in  STD_LOGIC;
      ALU1bit_MSB_opcode0  : in  STD_LOGIC;
      ALU1bit_MSB_opcode1  : in  STD_LOGIC;
		ALU1bit_MSB_binvert  : in  STD_LOGIC;
		ALU1bit_MSB_carryin  : in  STD_LOGIC;
      ALU1bit_MSB_result   : out STD_LOGIC;
      ALU1bit_MSB_carryout : out STD_LOGIC;
      ALU1bit_MSB_overflow : out STD_LOGIC
	);
end ALU1bit_MSB;

architecture Behavioral of ALU1bit_MSB is

--COMPONENTS
component AND2 is
	Port ( 
		a      : in  STD_LOGIC;
	   b      : in  STD_LOGIC;
	   result : out STD_LOGIC
	);
end component AND2;

component OR2 is
	Port ( 
		a      : in  STD_LOGIC;
	   b      : in  STD_LOGIC;
	   result : out STD_LOGIC
	);
end component OR2;

component XOR2 is
	Port ( 
		a      : in  STD_LOGIC;
	   b      : in  STD_LOGIC;
	   result : out STD_LOGIC
	);
end component XOR2;

component ADDER2 is
	Port ( 
		a     : in  STD_LOGIC;
	   b     : in  STD_LOGIC;
		sum   : out STD_LOGIC;
		carry : out STD_LOGIC
	);
end component ADDER2;

component MULT2to1 is
	Port ( 
		a      : in  STD_LOGIC;
	   b      : in  STD_LOGIC;
		sel    : in  STD_LOGIC;
	   result : out STD_LOGIC
	);
end component MULT2to1;

component MULT4to1 is
	Port ( 
		a      : in  STD_LOGIC;
	   b      : in  STD_LOGIC;
	   c      : in  STD_LOGIC;
	   d      : in  STD_LOGIC;
		sel0   : in  STD_LOGIC;
		sel1   : in  STD_LOGIC;
	   result : out STD_LOGIC
	);
end component MULT4to1;

--SIGNALS
signal andgate_out     : std_logic;
signal orgate_out      : std_logic;
signal adder_sum_out   : std_logic;
signal adder_carry_out : std_logic;
signal mux2to1_out     : std_logic;  

begin

--PORT MAP mux21
mux21: MULT2to1 port map(
	
	a      => ALU1bit_MSB_b,
	b      => not ALU1bit_MSB_b,
	sel    => ALU1bit_MSB_binvert,
	result => mux2to1_out
	
);

--PORT MAP andgate
andgate: AND2 port map(
	
	a      => ALU1bit_MSB_a,
	b      => mux2to1_out,
	result => andgate_out
	
);

--PORT MAP orgate
orgate: OR2 port map(
	
	a      => ALU1bit_MSB_a,
	b      => mux2to1_out,
	result => orgate_out
	
);

--PORT MAP adder
adder: ADDER2 port map(
	
	a     => ALU1bit_MSB_a,
	b     => mux2to1_out,
	sum   => adder_sum_out,
	carry => adder_carry_out
	
);

--PORT MAP mux41
mux41: MULT4to1 port map(
	
	a      => andgate_out,
	b      => orgate_out,
	c      => adder_sum_out,
	d      => ALU1bit_MSB_less,
	sel0   => ALU1bit_MSB_opcode0,
	sel1   => ALU1bit_MSB_opcode1,
	result => ALU1bit_MSB_result
	
);

--PORT MAP xorgate
xorgate: XOR2 port map(
	
	a      => adder_carry_out,
	b      => ALU1bit_MSB_carryin,
	result => ALU1bit_MSB_overflow
	
);

--ASSIGN OVERFLOW W/ DIRECT LOGIC
ALU1bit_MSB_carryout <= adder_carry_out;

end Behavioral;

